#ifndef __DFONTGL_H__
#define __DFONTGL_H__

#include "DFont.h"
#include <vector>

class DFontGL : public DFont {
public:
  DFontGL();
	DFontGL(const char *IFile, const char *DFile, int screenWidth_i, int screenHeight_i);
	~DFontGL();

  // load a font
  void load(const char *file);
  // this one should be called after every resolution change
  void setScreenSize(int screenWidth_i, int screenHeight_i);
  // save OpenGL state and prepare for putting strings
  static void prepareState();
  // restore openGL state
  static void restoreState();
  // build textures (upload them to video memory and set their OpenGL parameters)
  // textures are usually built internally, this function is provided for cases
  // that need a reinitialization (for example after a window resize)
  void buildTextures();

  void PutString(Sint16 x, Sint16 y, const char *str);
  void PutChar(Sint16 x, Sint16 y, const char c);
  void PutInt(Sint16 x, Sint16 y, int num);

private:
  class texture {
  public:
    SDL_Surface *surface;
    unsigned int id;
    int fontSurfaceStart;

    texture();
    ~texture();
    void alloc(int w, int h);
    void build();
    void bind();
  };

  class character {
  public:
    texture *t;
    float u1, v1, u2, v2;
    float w, h;

    character();
    void set(texture *t_i, float u1_i, float v1_i, float u2_i, float v2_i, float w_i, float h_i);
    void render(float x, float y);
  };

  const int maxTextureWidth;
  const int maxTextureHeight;

  // make textures from fontSurface and upload them
  void makeTextures();
  // get texture in wich the given point resides
  int getTexture(int x);
  // get uv coordinates of a point (x, y in fontSurface) in the given texture
  void getUV(int textX, int x, int y, float &u, float &v);
//public:
  vector<texture> textures;

  // make font map from DFont::map
  void makeMap();
  map<char, character> _fonts;

  // static variables used to set/restore OpenGL state
  static bool stateSet;
  static bool glTexture2DVal;
  static bool glBlendVal;
  static int glBlendSrcVal;
  static int glBlendDstVal;
  static int screenWidth;
  static int screenHeight;
};

#endif
